@@ -7,22 +7,24 @@
 #' This is needed as input for \code{\link{tdmBigLoop}}.
 #'
 #' @param tdm   a list with general settings for TDMR, see \code{\link{tdmDefaultsFill}}  
+#' @param sCList [defaultSC()] a list of list with controls for SPOT or other tuners (one list
+#'        for each element in \code{tdm$runList})  
 #'
 #' @return Environment \code{envT},  an object of class \code{\link{TDMenvir}},  containing (among others) the elements
 #'      \item{\code{runList}}{ \code{=tdm$runList}  }
-#'      \item{\code{spotList}}{ \code{=tdm$spotList}  }
+#      \item{\code{spotList}}{ \code{=tdm$spotList}  }
 #'      \item{\code{tdm}}{ \code{=\link{tdmDefaultsFill}(tdm)}  }
 #'      \item{\code{getBst}}{ accessor function(confFile,nExp,theTuner) into \code{envT$bstGrid}   }
 #'      \item{\code{getRes}}{ accessor function(confFile,nExp,theTuner) into \code{envT$resGrid}   }
 #'      \item{\code{sCList}}{ list of spotConfig-objects, as many as \code{envT$runList} has elements. Each spotConfig object 
-#'          sCList[[k]] contains a list \code{opts} as element, which is read from .apd file specified in \code{envT$runList[k]}.  }
+#'          \code{sCList[[k]]} contains a list \code{opts} as element for the machine learning part.  }
 #'
 #' @seealso   \code{\link{tdmBigLoop}}
-#' @author Wolfgang Konen (\email{wolfgang.konen@@th-koeln.de}), Patrick Koch
+#' @author Wolfgang Konen (\email{wolfgang.konen@@th-koeln.de}), THK, Patrick Koch
 #' @aliases TDMenvir 
 #' @export
 ######################################################################################
-tdmEnvTMakeNew <- function(tdm=NULL) {
+tdmEnvTMakeNew <- function(tdm=NULL, sCList=defaultSCList()) {
   #tdm$optsPath=tdm$path
   ######################################################################################
   # helper fcts for tdmEnvTMakeNew:
@@ -46,6 +48,7 @@ tdmEnvTMakeNew <- function(tdm=NULL) {
         }
         wP;
   }
+  #
   # function addSRF adds to opts the element opts$srfFile and reads in case opts$SRF.calc==FALSE from this file the
   # element opts$srf (a list with as many data frames as there are response variables in the task of confFile)
   addSRF <- function(confFile,opts) {
@@ -64,6 +67,18 @@ tdmEnvTMakeNew <- function(tdm=NULL) {
       }
       opts;
   }
+  #
+  # function runListFromSC extracts from sCList a vector of configuration names:
+  # either elements sCName or generic c("C01", "C02", ...)
+  runListFromSC <- function(sCList,tdm) {
+    if (!is.null(tdm$runList)) return(tdm$runList);
+    
+    runList <- sprintf("C%02d.conf",seq(length(sCList)));
+    for (i in 1:length(sCList)) 
+      if (!is.null(sCList[[i]]$sCName)) runList[i] <- sCList[[i]]$sCName;
+      
+    runList;
+  }
   ######################################################################################
   # end helper fcts for tdmEnvTMakeNew
   ######################################################################################
@@ -74,9 +89,11 @@ tdmEnvTMakeNew <- function(tdm=NULL) {
   envT$roiGrid <- list();
   envT$sCList <- list();
   envT$theFinals <- NULL;
+  tdm$runList <- runListFromSC(sCList,tdm);     # NEW in TDMR2.0
   envT$tdm <- tdm <- tdmDefaultsFill(tdm);
+  print(envT$tdm$path);
   envT$runList <- tdm$runList;
-  envT$spotList <- tdm$spotList;
+  envT$spotList <- tdm$spotList; # deprecated
   envT$wP <- ifelse(is.null(tdm$withParams), length(tdm$runList)==1, tdm$withParams)
   if (is.null(tdm$runList)) stop("tdm$runList is NULL");
   if (is.null(tdm$optsPath)) tdm$optsPath="./"
@@ -91,39 +108,28 @@ tdmEnvTMakeNew <- function(tdm=NULL) {
   firstRoiNames <- NULL;    # private storage for checkRoiParams
 
   #
-  # do all necessary file reading **before**  branching into tdmBigLoop 
-  # (bigLoopStep in tdmBigLoop.r can be in parallel execution branch, where file access might be not possible)
+  # check and extend settings in sCList **before** branching into tdmBigLoop 
   #
   k=0;
   for (confFile in envT$runList) {
     k=k+1;
-    lastChar=substr(tdm$path,nchar(tdm$path),nchar(tdm$path))
-    sepChar=ifelse(lastChar=="/","","/")
-    pathConfFile = paste(tdm$path,confFile,sep=sepChar);
-    if (!file.exists(pathConfFile)) stop(sprintf("Could not find confFile=%s (current dir=%s)",pathConfFile,getwd()));
-    envT$sCList[[k]] <- spotGetOptions(srcPath=tdm$theSpotPath,pathConfFile);
-    sC <- envT$sCList[[k]]; print(sC$spot.seed);
+    #--- not necessary anymore in TDMR2.0
+    #lastChar=substr(tdm$path,nchar(tdm$path),nchar(tdm$path))
+    #sepChar=ifelse(lastChar=="/","","/")
+    #pathConfFile = paste(tdm$path,confFile,sep=sepChar);
+    #if (!file.exists(pathConfFile)) stop(sprintf("Could not find confFile=%s (current dir=%s)",pathConfFile,getwd()));
+    #---
+    envT$sCList[[k]] <- sCList[[k]]; # spotGetOptions(srcPath=tdm$theSpotPath,pathConfFile);
+    sC <- envT$sCList[[k]]; print(sC$seedSPOT);
     envT$wP <- checkRoiParams(envT$wP,confFile,sC,envT);
-    pdFile = sC$io.apdFileName;
-  	print(pdFile);
-    pdFile = paste(tdm$path,pdFile,sep=sepChar);
-    if (!file.exists(pdFile)) stop(sprintf("Could not find pdFile=%s (current dir=%s)",pdFile,getwd()));
-    opts <- NULL;     # just to make 'R CMD check' happy  (and in case that after sourcing pdFile 'opts' is not there as expected)
-  	source(pdFile,local=TRUE);        # read problem design  (here: all elements of list opts)
-		if (is.null(opts))
-		  stop(sprintf("%s does not define the required object opts.",pdFile));
+    opts <- sCList[[k]]$opts;
     if (class(opts)[1]!="tdmOpts") 
       warning("Object opts is not of class tdmOpts. Consider constructing opts with tdmOptsDefaultsSet().");
-    #opts$path=tdm$optsPath;
-    #if (tdm$parallelCPUs>1 & sC$spot.fileMode) {
-    #  warning(sprintf("%s: Should not have spot.fileMode==TRUE in parallel execution. spot.fileMode is set to FALSE.",confFile));
-    #  envT$sCList[[k]]$spot.fileMode=FALSE;
-    #}
     if (!is.null(tdm$TST.trnFrac)) opts$TST.trnFrac=tdm$TST.trnFrac;
     if (!is.null(tdm$TST.valiFrac)) opts$TST.valiFrac=tdm$TST.valiFrac;
     if (!is.null(tdm$READ.target)) opts$READ.target=tdm$READ.target;
     if (!is.null(tdm$oFileMode)) opts$fileMode=tdm$oFileMode;
-
+ 
     opts=tdmOptsDefaultsSet(opts,path=tdm$path);
     opts=addSRF(confFile,opts);      # add opts$srfFile and add opts$srf from file in case opts$SRF.calc==FALSE
     checkOpts(opts);
@@ -141,58 +147,7 @@ tdmEnvTMakeNew <- function(tdm=NULL) {
   envT;
 }
 
-######################################################################################
-# tdmEnvTReadApd:
-#
-#' Re-read APD-files (list opts) for an environment envT of class \code{\link{TDMenvir}}.
-#'
-#' Given an already existing envT with envT$sClist filled, re-read the opts settings from APD files whose filenames 
-#' are specified in envT$sClist[[k]]$io.apdfilename.
-#'
-#' @param envT  an environment of class \code{\link{TDMenvir}}
-#' @param tdm   a list with general settings for TDMR (can be envT$tdm)  
-#'
-#' @return modified environment \code{envT},  an object of class \code{\link{TDMenvir}},  where the items
-#'      \code{envT$sCList[[k]]$opts} are modified for all \code{k}        
-#'
-#' @seealso   \code{\link{tdmEnvTMakeNew}}
-#' @author Wolfgang Konen (\email{wolfgang.konen@@th-koeln.de}), Patrick Koch
-#' @export
-#' @keywords internal
-######################################################################################
-tdmEnvTReadApd <- function(envT,tdm) {
-  k=0;
-  for (confFile in envT$runList) {
-    k=k+1;
-    sC <- envT$sCList[[k]]; 
-    pdFile = sC$io.apdFileName;
-  	print(pdFile);
-    pdFile = paste(tdm$path,pdFile,sep="");
-    if (!file.exists(pdFile)) stop(sprintf("Could not find pdFile=%s (current dir=%s)",pdFile,getwd()));
-    opts <- NULL;     # just to make 'R CMD check' happy  (and in case that after sourcing pdFile 'opts' is not there as expected)
-  	source(pdFile,local=TRUE);        # read problem design  (here: all elements of list opts)
-		if (is.null(opts))
-		  stop(sprintf("%s does not define the required object opts.",pdFile));
-    if (class(opts)[1]!="tdmOpts") 
-      warning("Object opts is not of class tdmOpts. Consider constructing opts with tdmOptsDefaultsSet().");
-    if (!is.null(tdm$TST.trnFrac)) opts$TST.trnFrac=tdm$TST.trnFrac;
-    if (!is.null(tdm$TST.valiFrac)) opts$TST.valiFrac=tdm$TST.valiFrac;
-    if (!is.null(tdm$READ.target)) opts$READ.target=tdm$READ.target;
-    if (!is.null(tdm$oFileMode)) opts$fileMode=tdm$oFileMode;
-
-    opts=tdmOptsDefaultsSet(opts,path=tdm$path);
-    checkOpts(opts);
-
-    if (tdm$umode[1]=="TST" & opts$TST.kind=="col" & !(opts$MOD.method %in% c("RF","MC.RF")))
-      warning(sprintf("%s: Do you really want tdm$umode=='TST' and opts$TST.kind=='col' when opts$MOD.method is not based on RF? %s",
-                      confFile,"You will have no validation data!"));
- 	  envT$sCList[[k]]$opts=opts;
-  }
-  
-  envT; 
-}
-
-# function checkOpts checks whether there are any new variable names in list opts (e.g. due to misspelling in APD file)
+# function checkOpts checks whether there are any new variable names in list opts (e.g. due to misspelling in opts-construction)
 # and if so, issues a NOTE and a warning message.
 checkOpts <- function(opts) {
   availNames = c("APPLY_TIME","CLS.CLASSWT","CLS.cutoff","CLS.gainmat","data.title","dir.data","dir.output","dir.Rdata"
@@ -204,7 +159,7 @@ checkOpts <- function(opts) {
                 ,"rgain.string","rgain.type","srf","SRF.calc","SRF.cutoff","SRF.kind","SRF.maxS","SRF.method","SRF.minlsi","SRF.ndrop"
                 ,"SRF.nkeep","SRF.ntree","SRF.samp","SRF.scale","SRF.verbose","SRF.XPerc","srfFile","SVM.coef0","SVM.cost","SVM.degree","SVM.epsilon"
                 ,"SVM.gamma","SVM.kernel","SVM.tolerance","ADA.coeflearn","ADA.mfinal","ADA.rpart.minsplit","test2.show","test2.string"
-                ,"TST.COL","TST.kind","TST.NFOLD","TST.SEED","TST.trnFrac","TST.valiFrac","VERBOSE");
+                ,"TST.COL","TST.kind","TST.NFOLD","TST.SEED","TST.trnFrac","TST.valiFrac","VERBOSE","path");
   newNames = setdiff(names(opts),availNames);
   if (length(newNames)>0) {
     if (length(newNames)==1) cat("NOTE: A new variable has been defined for list opts: ",newNames,".\n");
@@ -257,20 +212,50 @@ tdmEnvTAddGetters <- function(envT) {
 #'
 #' The loaded envT is augmented with getter functions, see \code{\link{tdmEnvTAddGetters}}.
 #'
-#' @param   fileRData   string with filename to load. This file is searched in the working dir.
+#' @param   fileRData   string with filename to load. 
+#' @param   path   [NULL] dir where to search \code{fileRData}. If NULL, use current dir.
 #' @return  envT
 #' @export
 ######################################################################################
-tdmEnvTLoad <- function(fileRData) {
-  load(fileRData); # loads envT
+tdmEnvTLoad <- function(fileRData, path=NULL) {
+  pathFile = fileRData;
+  if (!is.null(path)) pathFile = paste(path,fileRData,sep="/");
+  load(pathFile); # loads envT
   envT <- tdmEnvTAddGetters(envT);
   class(envT) <- c("TDMenvir","environment");
   envT;
 }
 
+######################################################################################
+#  tdmEnvTUpdate:
+#' Update \code{envT$tdm}
+#'
+#' Update \code{envT$tdm} with the non-NULL elements of \code{tdm}
+#' 
+#' @param   envT environment TDMR
+#' @param   tdm  list for TDMR, see \code{\link{tdmDefaultsFill}}
+#' @return  envT
+#' @export
+######################################################################################
+tdmEnvTUpdate <- function(envT,tdm) {
+  if(methods::hasArg(tdm)){
+    matching<-intersect(names(tdm),names(envT$tdm))
+    envT$tdm[matching] <- tdm[matching]
+    
+    #notMatching <- setdiff(names(tdm),
+    #                       names(envT$tdm))
+    #if(length(notMatching)!=0) warning(paste("The following arguments in tdm are ignored: ", notMatching))
+    #### 
+    #### We exclude this test here, because there are members in envT$tdm with default value NULL.  
+    #### Each of them would trigger a warning if it were defined in argument tdm.
+  }
+  
+  envT;
+}
+
 ######################################################################################
 #  tdmEnvTGetOpts:
-#' Return list opts from the \code{k}-th \code{.conf}-file
+#' Return list opts from the \code{k}-th element of \code{envT$sCList}
 #'
 #' @param   envT environment TDMR
 #' @param   k    [1] index 1,...,length(envT$runList)
@@ -283,7 +268,7 @@ tdmEnvTGetOpts <- function(envT,k=1) {
 
 ######################################################################################
 #  tdmEnvTSetOpts:
-#' Set list opts for the \code{k}-th \code{.conf}-file
+#' Set list opts for the \code{k}-th element of \code{envT$sCList}
 #'
 #' @param   envT environment TDMR
 #' @param   opts list of options
@@ -328,32 +313,33 @@ tdmEnvTAddBstRes <- function(envT,fileRData) {
 }
 
 ######################################################################################
-#  tdmEnvTSensi:
-#' Make a sensitivity plot based on \code{envT}
+#  tdmEnvTReport:
+#' Make a report plot based on \code{envT}
 #'
 #' Given the results from a prior tuning run in \code{envT}, make a sensitivity plot
-#' for this run (call SPOT with spotStep="rep")
+#' for this run. \cr
 #' If \code{envT$tdm$nrun > 0} then make additionally with the best-performing parameters from 
 #' the tuning run a new unbiased run on the test data.
 #'
-#' @param   envT  results from a prior tuning run. If envT$tdm$run==0
+#' @param   envT  results from a prior tuning run. 
 #' @param   ind   an integer from \code{1:length(envT$bstGrid)}: Take the tuning run with index \code{ind}.
-#' @return  \code{finals}, either NULL (no unbiased run) or a one-row data frame with the 
-#'          results from the unbiased run
+#' @return  \code{envT}, with data frame \code{finals} added, if  \code{envT$tdm$nrun > 0}.
 #' @examples            
 #'    ## The best results are read from demo02sonar/demoSonar.RData relative to the TDMR 
 #'    ## package directory.
-#'    oldwd <- getwd(); setwd(paste(find.package("TDMR"), "demo02sonar",sep="/"));
-#'    envT = tdmEnvTLoad("demoSonar.RData");    # loads envT
-#'    source("main_sonar.r");
+#'    path = paste(find.package("TDMR"), "demo02sonar",sep="/");
+#'    envT = tdmEnvTLoad("demoSonar.RData",path);    # loads envT
+#'    source(paste(path,"main_sonar.r",sep="/"));
 #'    envT$tdm$nrun=0;       # =0: don't, >0: do unbiasedRun with opts$NRUN=envT$tdm$nrun
-#'    finals = tdmEnvTSensi(envT,1);
-#'    if (!is.null(finals)) print(finals);
-#'    setwd(oldwd);
+#'    envT$sCList[[1]]$opts$VERBOSE=1;
+#'    envT <- tdmEnvTReport(envT,1);
+#'    if (!is.null(envT$theFinals)) print(envT$theFinals);
 #'
+#' @seealso  \code{\link{tdmEnvTReportSens}} 
 #' @export
-######################################################################################
-tdmEnvTSensi <- function(envT,ind) {
+##############################################################################################
+#--- currently only sensitivity-based report, but other reports are later possible as well ---
+tdmEnvTReport <- function(envT,ind) {
   tdm = envT$tdm;
   if (is.null(tdm$runList)) {
     if (is.null(envT$runList)) stop("Both envT$runList and envT$tdm$runList are NULL");
@@ -368,12 +354,16 @@ tdmEnvTSensi <- function(envT,ind) {
   theTuner = tuneVec[ind];
   nExp = expeVec[ind];
   confFile = confVec[ind];
-  nConf = which(confFile==envT$runList);
+  #nConf = which(confFile==envT$runList);
+  nConf = which(confFile==tdm$runList);
+  print(nConf);
+  if (length(nConf)==0) 
+    stop ("Something wrong with confFile and tdm$runList");
   #print(c(ind,confFile,nExp,theTuner));
   
-  correctEnvTDir(envT,tdm);                # correct direcotories in envT$sCList
+  #correctEnvTDir(envT,tdm);                # correct direcotories in envT$sCList
 
-  envT$spotConfig <- envT$sCList[[nConf]]; # spotGetOptions(srcPath=tdm$theSpotPath,confFile);  
+  envT$spotConfig <- envT$sCList[[nConf]];   
   envT$bst = envT$bstGrid[[ind]];
   envT$res = envT$resGrid[[ind]];
   #
@@ -387,65 +377,181 @@ tdmEnvTSensi <- function(envT,ind) {
   
   envT$spotConfig$spot.fileMode=FALSE;    # call SPOT with spot.fileMode=FALSE (take everything from  envT$spotConfig)
  
-  ptm <- proc.time();
+  # ptm <- proc.time();
   
-      # this is for the case spotStep=="rep":
-      envT$spotConfig$alg.currentResult <- envT$res;
-      envT$spotConfig$alg.currentBest <- envT$bst;	
-      
-      cat(sprintf("*** Starting SENS REPORT for tuner %s, spotStep=%s, on task %s ***\n",theTuner,"rep",confFile));
-
-      tdmDispatchTuner(theTuner,confFile,"rep",tdm,envT,dataObj);
-      # If spotStep=="rep" then tdmDispatchTuner expects that envT$bst, envT$res contain already the right data frames 
-      # and it runs just the reporting step of SPOT.
+  # this is needed for tdmEnvTReportSens:
+  envT$spotConfig$alg.currentResult <- envT$res;
+  envT$spotConfig$alg.currentBest <- envT$bst;	
+  
+  writeLines(sprintf("\n*** Starting SENS REPORT for tuner %s, on task %s ***",theTuner,confFile),con=stderr());
+  
+  tdmEnvTReportSens(envT$spotConfig);
+  
+  # if (is.null(tdm$timeMode)) stop("tdm$timeMode is not set (NULL). Consider 'tdm <- tdmDefaultsFill(tdm)' to set all defaults");
+  # envT$time.TRN=(proc.time()-ptm)[tdm$timeMode]; opts=list(); opts$VERBOSE=1;  
+  # cat1(opts,paste(time.txt[tdm$timeMode], "time for tuning with tdmDispatchTuner:",envT$time.TRN,"sec\n"));     
+  opts <- envT$spotConfig$opts;
   
-  time.txt = c("Proc", "System", "Elapsed");
-  if (is.null(tdm$timeMode)) stop("tdm$timeMode is not set (NULL). Consider 'tdm <- tdmDefaultsFill(tdm)' to set all defaults");
-  envT$time.TRN=(proc.time()-ptm)[tdm$timeMode]; opts=list(); opts$VERBOSE=1;  
-  cat1(opts,paste(time.txt[tdm$timeMode], "time for tuning with tdmDispatchTuner:",envT$time.TRN,"sec\n"));     
-
   ptm <- proc.time();
-  finals <- NULL;
+  envT$finals <- NULL;
   if (tdm$nrun>0) {
       #
       # read the data and split them into test data and train/vali data
-      dataObj <- tdmSplitTestData(envT$spotConfig$opts,tdm,nExp);
-      if (!is.null(dataObj)) envT$spotConfig$opts$TST.COL = dataObj$TST.COL;    # this column has to be subtracted in main_* from the input variables
+      dataObj <- tdmReadAndSplit(opts,tdm,nExp);
+      if (!is.null(dataObj)) opts$TST.COL = dataObj$TST.COL;    # this column has to be subtracted in main_* from the input variables
 
       if (!is.null(tdm$mainFile)) {
-        if (!file.exists(tdm$mainFile)) stop(sprintf("Could not find tdm$mainFile=%s (current dir=%s)",tdm$mainFile,getwd()));
- 	      source(tdm$mainFile);
+        pathFile <- paste(opts$path,tdm$mainFile,sep="/")
+        if (!file.exists(pathFile)) stop(sprintf("Could not find tdm$mainFile=%s (path=%s)",tdm$mainFile,opts$path));
+ 	      source(pathFile);
       }
     
       for (umode in tdm$umode) {
-        cat("*** starting",tdm$unbiasedFunc,"for",confFile,"with umode=",umode,"***\n");
-        cmd=paste("finals <-",tdm$unbiasedFunc,"(confFile,envT,dataObj,umode=umode,finals=finals,withParams=envT$wP,tdm=tdm)",sep="");
-        eval(parse(text=cmd));
+        writeLines(sprintf("\n*** Starting %s for %s with umode= %s ***",tdm$unbiasedFunc,confFile,tdm$umode),con=stderr());
+        # cmd=paste("envT <-",tdm$unbiasedFunc,"(confFile,envT,dataObj,umode=umode,withParams=envT$wP,tdm=tdm)",sep="");
+        # eval(parse(text=cmd));
+        envT <- eval(call(tdm$unbiasedFunc,confFile,envT,dataObj,umode=umode,withParams=envT$wP,tdm=tdm))
       }
+      time.txt = c("Proc", "System", "Elapsed");
       time.TST=(proc.time()-ptm)[tdm$timeMode];   
       cat1(opts,paste(time.txt[tdm$timeMode], "time for",tdm$unbiasedFunc,":",time.TST,"sec\n"));   
-      finals <- cbind(finals,Time.TST=time.TST);        
-      flush.console(); 
+      envT$finals <- cbind(envT$finals,Time.TST=time.TST);    
+      envT$theFinals <- envT$finals;
+      row.names(envT$theFinals) <- (1:nrow(envT$theFinals));   # get rid of "user.self"
+      flush.console();  
   } # if(tdm$nrun>0)
    
-  finals;
-} # end of function tdmEnvTSensi
+  envT;
+} # end of function tdmEnvTReport
 
 ######################################################################################
-# helper fct for tdmEnvTSensi:
+# --- DEPRECATED ---
+# helper fct for tdmEnvTReport:
 #     correct the path in directories of envT$sCList
-correctEnvTDir <- function(envT,tdm) {
-    if (!is.null(tdm$path)) {
-      for (i in 1:length(envT$sCList)) {    
-        # replace the tdm$path-part of opts$dir.* with "./". This workaround necessary when the 
-        # original tuning job was done on another machine, e.g. maanbs03, where the path is 
-        # expanded to /home/wolfgang/fiwa/... and we run this function on a Windows system
-        envT$sCList[[i]]$opts$dir.data = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.data);
-        envT$sCList[[i]]$opts$dir.txt = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.txt);
-        envT$sCList[[i]]$opts$dir.Rdata = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.Rdata);
-        envT$sCList[[i]]$opts$dir.output = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.output);
-      }     
-    }
+# correctEnvTDir <- function(envT,tdm) {
+#     if (!is.null(tdm$path)) {
+#       for (i in 1:length(envT$sCList)) {    
+#         # replace the tdm$path-part of opts$dir.* with "./". This workaround necessary when the 
+#         # original tuning job was done on another machine, e.g. maanbs03, where the path is 
+#         # expanded to /home/wolfgang/fiwa/... and we run this function on a Windows system
+#         envT$sCList[[i]]$opts$dir.data = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.data);
+#         envT$sCList[[i]]$opts$dir.txt = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.txt);
+#         envT$sCList[[i]]$opts$dir.Rdata = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.Rdata);
+#         envT$sCList[[i]]$opts$dir.output = sub(tdm$path,"./",envT$sCList[[i]]$opts$dir.output);
+#       }     
+#     }
+# }
+
+###################################################################################################
+#'  Sensitivity Report Helper Function
+#' 
+#' Helper function for the sensitivity report
+#'
+#' @param B    best solution column
+#' @param fit  random forest fit
+#' @param roi  internal parameter for the initial region of interest, or: \code{spotConfig$alg.roi}
+#' @param nsens number of parameters estimated with sensitivity
+#' @return Y
+#' 
+#' @seealso  \code{\link{tdmEnvTReport}}  \code{\link{tdmEnvTReportSens}} 
+#' @export
+#' @keywords internal
+################################################################################################### 
+tdmReportSensY <- function(B,fit,roi,nsens) {	
+  Y <- NULL
+  for (k in 1:length(B)) {
+    BV <- B;
+    BV[,k] <- seq(roi[k,"lower"],roi[k,"upper"],length.out=nsens)
+    Y <- data.frame(cbind(Y,predict(fit,BV)))
+    names(Y)[length(Y)] <- names(B)[k]
+  }	
+  Y
+}
+
+###################################################################################################
+# tdmEnvTReportSens: 
+#' 
+#' Function to generate a report with sensitivity plot.
+#'
+#' The sensitivity curves are based on a metamodel which is a random forest with 100 trees 
+#' fitted to the result points from RES-file. The plot contains: 
+#'    x-axis:   ROI for each parameter normalized to [-1,1]
+#'    y-axis: 
+#'
+#' @param spotConfig the configuration list of all spot parameters
+#' @seealso  \code{\link{tdmEnvTReport}} 
+#' @export
+################################################################################################### 
+tdmEnvTReportSens <- function(spotConfig) {	
+  #require(randomForest); # now via direct call 'randomForest::'
+  
+  opts <- spotConfig$opts;
+  cat2(opts,"  Entering tdmEnvTReportSens\n");
+  rawData=spotConfig$alg.currentResult;
+  # remove the columns which change over replicates:
+  rawData=rawData[,setdiff(names(rawData),c("REP","SEED"))];
+  print2(opts,summary(rawData));
+  
+  mergedData <- stats::aggregate(Y~.,data=rawData,FUN=mean);
+  C <- data.frame(mergedData[order(mergedData$Y,decreasing=FALSE),,drop=FALSE]);
+  best <-  C[1,,drop=FALSE];
+
+
+  #spotConfig=spotWriteBest(mergedData, spotConfig);
+  #C1=spotConfig$alg.currentBest[nrow(spotConfig$alg.currentBest),]       # WK: added ","
+  xNames <- row.names(spotConfig$alg.roi);
+  yNames <- spotConfig$alg.resultColumn;
+  B <- NULL; 
+  nsens=20;             # number of points along the normalized ROI range
+  for (i in 1:nsens) {
+    # replicate best solution (row 1), but cut away 1st column (Y):   
+    B <- rbind(B,data.frame(C[1,xNames]));
+  }
+  #browser()
+  names(B) <- xNames; 	
+  fit <- randomForest::randomForest(rawData[xNames], rawData[[yNames]], ntree=100)		
+  #fit <- rpart(y ~ ., data= rawB)
+  rwb <- cbind(spotConfig$alg.roi,t(B[1,]))     # rwb: roi with 'BEST' column
+  names(rwb)[length(rwb)] <- "BEST"
+  Y <- tdmReportSensY(B,fit,spotConfig$alg.roi,nsens)
+  # scale each ROI to the normalized ROI range [-1,+1]:
+  X=seq(-1,1,length.out=nsens)	
+  # XP is the location of the SPOT best point for each parameter in the normalized ROI range
+  XP = (rwb$BEST-rwb$lower)/(rwb$upper-rwb$lower)*2-1
+  XP=rbind(XP,XP);   # matpoints needs at least two rows to plot each column in a different color
+  YP = min(Y);  YP=rbind(YP,YP)
+  #palette("default")
+  cat1(opts," \n")
+  cat1(opts,"Sensitivity plot for this ROI:\n")
+  print1(opts,rwb)
+  cat1(opts," \n")
+  cat1(opts,paste("Best solution found with ",nrow(rawData)," evaluations:\n",sep=""))
+  print1(opts,C[1,])
+  cat1(opts," \n")
+  cat1(opts,"Standard deviation of best solution: ")
+  y<-rawData[rawData$CONFIG==C[1,"CONFIG"],spotConfig$alg.resultColumn]
+  cat1(opts,paste(mean(y),"+-",sd(y),"\n"))
+  #finally the plot commands, both for screen and pdf file
+  # if(spotConfig$report.io.pdf==TRUE){ #if pdf should be created
+  #   pdf(spotConfig$io.pdfFileName) #start pdf creation
+  #   matplot(X,Y,type="l",lwd=rep(3,ncol(Y)),cex.axis=1.5,cex.lab=1.5,col=1:ncol(Y),xlab="normalized ROI",main=spotConfig$userConfFileName) 
+  #   matpoints(XP,YP,pch=rep(21,ncol(Y)),bg=1:ncol(Y),cex=2)	
+  #   legend("topleft",legend=names(Y),lwd=rep(2,ncol(Y)),lty=1:ncol(Y),col=1:ncol(Y),text.col=1:ncol(Y))	
+  #   dev.off() #close pdf device
+  # }
+  # if(spotConfig$report.io.screen==TRUE && spotConfig$io.verbosity>0) #if graphic should be on screen
+  # {
+  #   dev.new()
+    graphics::matplot(X,Y,type="l",lwd=rep(3,ncol(Y)),cex.axis=1.5,cex.lab=1.5,col=1:ncol(Y),xlab="normalized ROI",main=spotConfig$userConfFileName) 
+    graphics::matpoints(XP,YP,pch=rep(21,ncol(Y)),bg=1:ncol(Y),cex=2)	
+    graphics::legend("topleft",legend=names(Y),lwd=rep(2,ncol(Y)),lty=1:ncol(Y),col=1:ncol(Y),text.col=1:ncol(Y))
+  #}
+  cat2(opts,"\n Leaving tdmEnvTReportSens\n");
+  
+  spotConfig;    # not really necessary, nothing changed in spotConfig
 }
 
 
+
+
+
